# ################################################################################
# Title: HATI monitor command script
# Project: HATI
#
# File       : monitor.sh
#
# Description: This script is used to execute the monitor command in the HATI shell through
# the JTAG to AXI module.
# 

# Licence and Copyright:
# Copyright (c) 2022 Safran Trusted 4D (https://www.safran-group.com/companies/safran-trusted-4d)
# 
# You might use, distribute and modify this code and its resulting
# binary form under the terms of the LICENSE.txt provided within the
# repository or project.
#
# If you have not received a copy of the LICENSE.txt along with
# this file please write to info<DOT>spain<AT>orolia<DOT>com and consider that
# this file can not be copied and/or distributed in any forms.
# ################################################################################

#!/bin/bash

TCL_LIB_PATH="./tcl_libs"
VIVADO_ARGS="-notrace -quiet -mode batch"

MODE_ARGUMENT="-mode"
ADDR_ARGUMENT="-addr"
XVC_MODE="xvc_no"
ip_and_port="none"
MODE="lite"
ADDR=0x1000000


bold="\e[1m"
normal="\e[0m"

UGUIDE='\e]8;;https://docs.google.com/document/d/1EE6IxLOZ7NMhykK6Oc6NYRJwYmJu1o1ohx5KgkAOSBA/edit?pli=1\aUser Guide\e]8;;\a'

clean_vivado_files() {
    rm -rf vivado*
    rm -rf .Xil
    rm -rf webtalk*
}

check_usage () {


    echo -e " \n ${bold}DESCRIPTION:\n${normal}"

    echo -e " The command monitor.sh prints information about the status of HATI, such as synchronization\n     status, link, sw version, among others.\n"

    echo -e " \n ${bold}ARGUMENTS:\n${normal}"

    echo -e " -m\tIt specifies that amount of data to be printed. It should be <lite> or <expert>.\n          ${bold}lite${normal} by default. Check the ${bold}${UGUIDE}${normal} for more information.\n"

    echo -e " -a\tIt specifies the HATI base address to read from. It should be written in the\n          format 0x9ABCDEF0 (4 bytes hexadecimal). ${bold}0x00000000${normal} by default.\n"

    echo -e " -r\tIt specifies the IP and port of the remote JTAG, or the remote xvc server.\n"

    echo -e " -v\tUses the -v option for connecting to a remote XVC server. If you use this option, \n          you must specify an IP and port with the -r argument too.\n"

    echo -e " -h\tPrints this guide.\n"


    echo -e "\n\n ${bold}EXAMPLES:\n${normal}"

    echo -e " Having two HATIs at addresses 0x1000000 (HATI 1) and 0x2000000 (HATI 2), the commands:\n"

    echo -e " ./monitor.sh"
    echo -e "     will print the monitor lite from HATI 1.\n"

    echo -e " ./monitor.sh -m expert"
    echo -e "     will print the monitor expert from HATI 1.\n"

    echo -e " ./monitor.sh -m expert -a 0x2000000"
    echo -e "     will print the monitor expert from HATI 2.\n"

    echo -e " ./monitor.sh -m expert -r 192.168.1.150:3121"
    echo -e "     will print the monitor expert from HATI 1 connected through remote JTAG whose \n     IP is 192.168.1.150, and port 3121.\n"

    echo -e " ./monitor.sh -m expert -a 0x2000000 -r 192.168.1.210:2542 -v"
    echo -e "     will print the monitor expert from HATI 2 connected through remote XVC server \n     whose IP is 192.168.1.210, and port 2542.\n"

}


# Get argument values
while getopts ":m:a:r:vh" opt; do
    case "${opt}" in
        m)
            MODE=${OPTARG}
            ;;
        a)
            ADDR=${OPTARG}
            ;;
        r)
            ip_and_port=${OPTARG}
            ;;
        v)
            XVC_MODE="xvc_yes"
            ;;
        h)
            check_usage
            clean_vivado_files
            exit
            ;;
        *)
            echo -e "\nYou enterered an invalid argument or configuration."
            check_usage
            clean_vivado_files
            exit
            ;;
    esac
done

echo -e "\nFor information and examples type: \e[4m./monitor.sh -h\n\e[0m"

# Check if mandatory arguments are provided
if test "$XVC_MODE" == "xvc_yes" && test "$ip_and_port" == "none"; then
    echo -e "If you use the -v option, you must specify an IP and port with the -r argument too.\n"
    exit
fi
if ([ -z "${ADDR}" ]); then
    check_usage
fi


vivado ${VIVADO_ARGS} -source ${TCL_LIB_PATH}/main.tcl -tclargs ${MODE_ARGUMENT} ${MODE} ${ADDR_ARGUMENT} ${ADDR} ${ip_and_port} ${XVC_MODE}

clean_vivado_files
