# ################################################################################
# Title: HATI memory map
# Project: HATI
#
# File       : memmap.tcl
#
# Licence and Copyright:
# Copyright (c) 2022 Safran Trusted 4D (https://www.safran-group.com/companies/safran-trusted-4d)
#
# You might use, distribute and modify this code and its resulting
# binary form under the terms of the LICENSE.txt provided within the
# repository or project.
#
# If you have not received a copy of the LICENSE.txt along with
# this file please write to info<DOT>spain<AT>orolia<DOT>com and consider that
# this file can not be copied and/or distributed in any forms.
# ################################################################################

# ================================
# Sources
# ================================

# Includes type-conversion functions
source ./tcl_libs/typeutils.tcl; list
# Includes HW utils
source ./tcl_libs/hwutils.tcl; list



# ================================
# Global Variables
# ================================

# Global parameters
set HATI_OFFSET_POINTER   0x1FF94
set RELADDR_PARAM_ADDR    [expr $::hati_addr + $HATI_OFFSET_POINTER]

# Reads the relative address from the "hati_ADDR" register
set RELATIVE_ADDR         [expr $::hati_addr + 0x[eval read_from $RELADDR_PARAM_ADDR]]

# HATI Version information
set ::COMMIT_ID_ADDR      [expr ${RELATIVE_ADDR} + 0x00]
set ::BUILD_TIME_ADDR     [expr ${RELATIVE_ADDR} + 0x20]
set ::BUILD_DATE_ADDR     [expr ${RELATIVE_ADDR} + 0x30]
set ::BUILD_BY_ADDR       [expr ${RELATIVE_ADDR} + 0x40]

# General parameters
set ::MODE_ADDR           [expr ${RELATIVE_ADDR} + 0x60]
set ::MAC_ADDR            [expr ${RELATIVE_ADDR} + 0xE0]
set ::UPTIME_ADDR         [expr ${RELATIVE_ADDR} + 0xE8]
set ::DACVAL_ADDR         [expr ${RELATIVE_ADDR} + 0xEC]
set ::TEMP_ADDR           [expr ${RELATIVE_ADDR} + 0xF0]

# Connected SFP parameters
set ::PN_ADDR               [expr ${RELATIVE_ADDR} + 0xF4]
set ::ALPHA_ADDR            [expr ${RELATIVE_ADDR} + 0x104]
set ::DTX_ADDR              [expr ${RELATIVE_ADDR} + 0x108]
set ::DRX_ADDR              [expr ${RELATIVE_ADDR} + 0x10C]
set ::CHECKSUM_ADDR         [expr ${RELATIVE_ADDR} + 0x110]
set ::EEPROM_DETECTED_ADDR  [expr ${RELATIVE_ADDR} + 0x114]

# Syncrhronization information
set ::SEC_ADDR                [expr ${RELATIVE_ADDR} + 0x64]
set ::LINK_STATUS_ADDR        [expr ${RELATIVE_ADDR} + 0x68]
set ::RX_PACKETS_ADDR         [expr ${RELATIVE_ADDR} + 0x6C]
set ::TX_PACKETS_ADDR         [expr ${RELATIVE_ADDR} + 0x70]
set ::LOCKED_ADDR             [expr ${RELATIVE_ADDR} + 0x74]
set ::SERVO_STATE_NAME_ADDR   [expr ${RELATIVE_ADDR} + 0x7C]
set ::CALIBRATED_ADDR         [expr ${RELATIVE_ADDR} + 0x78]
set ::SERVO_ADDR              [expr ${RELATIVE_ADDR} + 0x9C]
set ::TRACK_PHASE_ADDR        [expr ${RELATIVE_ADDR} + 0xA0]
set ::RTT_ADDR                [expr ${RELATIVE_ADDR} + 0xA4]
set ::MS_DELAY_ADDR           [expr ${RELATIVE_ADDR} + 0xA8]
set ::MANUAL_SHIFT_ADDR       [expr ${RELATIVE_ADDR} + 0x118]
set ::DELTA_TX_M_ADDR         [expr ${RELATIVE_ADDR} + 0xAC]
set ::DELTA_RX_M_ADDR         [expr ${RELATIVE_ADDR} + 0xB0]
set ::DELTA_TX_S_ADDR         [expr ${RELATIVE_ADDR} + 0xB4]
set ::DELTA_RX_S_ADDR         [expr ${RELATIVE_ADDR} + 0xB8]
set ::SEMISTATIC_RX_ADDR      [expr ${RELATIVE_ADDR} + 0xBC]
set ::T24P_DELAY_ADDR         [expr ${RELATIVE_ADDR} + 0xC0]
set ::LINK_ASYMMETRY_ADDR     [expr ${RELATIVE_ADDR} + 0xC4]
set ::CABLE_RTT_ADDR          [expr ${RELATIVE_ADDR} + 0xC8]
set ::CLOCK_OFFSET_ADDR       [expr ${RELATIVE_ADDR} + 0xCC]
set ::CURRENT_OFFSET_ADDR     [expr ${RELATIVE_ADDR} + 0xD0]
set ::MASTER_CLOCK_CLASS_ADDR [expr ${RELATIVE_ADDR} + 0xD4]
set ::SKEW_ADDR               [expr ${RELATIVE_ADDR} + 0xD8]
set ::UPDATE_COUNT_ADDR       [expr ${RELATIVE_ADDR} + 0xDC]

# HATI PTP announce parameters.
set ::HATI_CURRENT_UTC_OFFSET_ADDR        [expr ${RELATIVE_ADDR} + 0x128]
set ::HATI_CURRENT_UTC_OFFSET_VALID_ADDR  [expr ${RELATIVE_ADDR} + 0x12C]
set ::CLOCK_ID_ADDR                       [expr ${RELATIVE_ADDR} + 0x148]
set ::HATI_STEPS_REMOVED_ADDR             [expr ${RELATIVE_ADDR} + 0x150]

# Servo state name list. Access with servo state.
set ::SERVO_STATES {
    "UNINITIALIZED"
    "SYNC_NSEC"
    "SYNC_TAI"
    "SYNC_PHASE"
    "TRACK_PHASE"
    "WAIT_OFFSET_STABLE"
}


# ================================
# Functions
# ================================

# ###############################################################
# Name: read_mode.
#
# Description: Reads the HATI mode
#
# Parameters: None.
#
# Return: <string> HATI Mode:
#    - Slave.
#	 - Master.
# ###############################################################
proc read_mode {} {

    # Read HATI Mode
    set hati_mode [eval read_from ${::MODE_ADDR} "int"]

    # Return if the HATI is working as a Master or as a Slave
    if { ${hati_mode} == 1 } {
        return "Master"
    } else {
        return "Slave"
    }
}

# ###############################################################
# Name: read_hati_core_build.
#
# Description: Reads information about the GIT version and the
#			   core build.
#
# Parameters: None.
#
# Return: <string> Core Build:
#	- GIT version and core build.
# ###############################################################
proc read_hati_core_build {} {
    return [eval read_sequence ${::COMMIT_ID_ADDR} "str" 8]
}


# ###############################################################
# Name: read_build.
#
# Description: Reads information about the time, date and the
#			   author of the current build.
#
# Parameters: None.
#
# Return: <string> Build Information:
#	- Date, time and author of the build
# ###############################################################
proc read_build {} {

    set build_time [eval read_sequence ${::BUILD_TIME_ADDR} "str" 4]
    set build_date [eval read_sequence ${::BUILD_DATE_ADDR} "str" 4]
    set build_by   [eval read_sequence ${::BUILD_BY_ADDR}   "str" 8]

    return "${build_date} ${build_time} by ${build_by}"
}

# ###############################################################
# Name: read_mac.
#
# Description: Reads the HATI MAC address.
#
# Parameters: None.
#
# Return: <string> MAC address with format xx:xx:xx:xx:xx:xx
# ###############################################################
proc read_mac {} {

    # # Read MAC high address
    set hati_mac_h [eval read_from [expr ${::MAC_ADDR} + 0x00] "lts"]
    # # Read MAC low address
    set hati_mac_l [eval read_from [expr ${::MAC_ADDR} + 0x04] "lts"]

    # Create a list with each read character
    set mac_aux [concat $hati_mac_l $hati_mac_h]


    # Separate MAC bytes
    set macb1 "[lindex $mac_aux 4][lindex $mac_aux 5]"
    set macb2 "[lindex $mac_aux 6][lindex $mac_aux 7]"
    set macb3 "[lindex $mac_aux 8][lindex $mac_aux 9]"
    set macb4 "[lindex $mac_aux 10][lindex $mac_aux 11]"
    set macb5 "[lindex $mac_aux 12][lindex $mac_aux 13]"
    set macb6 "[lindex $mac_aux 14][lindex $mac_aux 15]"

    return "$macb6:$macb5:$macb4:$macb3:$macb2:$macb1";
}


# ###############################################################
# Name: read_uptime.
#
# Description: Reads the time the HATI has been active.
#
# Parameters: None.
#
# Return: <unsigned> Uptime.
# ###############################################################
proc read_uptime {} {
    return [eval read_from ${::UPTIME_ADDR} "int"]
}


# ###############################################################
# Name: read_sfp_plugged.
#
# Description: Reads the SFP information if connected:
#              Part Number, Alpha offset, TX and RX delays and
# 			   the checksum.
#
# Parameters: None.
#
# Return: <string>
#	 - If the SFP is connected: returns the SFP information.
#    - If the SFP isn't connected: returns "SFP not plugged".
# ###############################################################
proc read_sfp_plugged {} {

    set alpha [eval read_from ${::ALPHA_ADDR} "int"]

    if { ${alpha} == -1 } {
        return "SFP not plugged";
    }

    set pn [eval read_sequence ${::PN_ADDR} "str" 4]

    set dtx_hex 	[eval read_from ${::DTX_ADDR} "int"]
    set drx_hex 	[eval read_from ${::DRX_ADDR} "int"]
    set checksum 	[eval read_from ${::CHECKSUM_ADDR} "int"]

    set dtx_int 	[eval read_from ${::DTX_ADDR} "int"]
    set drx_int 	[eval read_from ${::DRX_ADDR} "int"]

    return  "\nSFP Plugged:
   Part Number: ${pn}
   TX Delay:    ${dtx_int} (${dtx_hex})
   RX Delay:    ${drx_int} (${drx_hex})
   Checksum:    ${checksum}\n"
}

# ###############################################################
# Name: read_tai.
#
# Description: Reads unformated TAI time.
#
# Parameters: None.
#
# Return: <unsigned>: TAI time in seconds.
# ###############################################################
proc read_tai {} {
    return [eval read_from ${::SEC_ADDR} "int"]
}

# ###############################################################
# Name: read_link_status.
#
# Description: Reads unformated TAI time.
#
# Parameters: None.
#
# Return: <unsigned>: Status of the link:
#	 - 0: It exist a link in the current HATI.
#    - 1: It doesn't exist a link in the current HATI.
# ###############################################################
proc read_link_status {} {
    return [eval read_from ${::LINK_STATUS_ADDR} "int"]
}


# ###############################################################
# Name: read_packets.
#
# Description: Reads the number of packets of the TX and RX.
#
# Parameters: None.
#
# Return: <string>: Packages of the TX and RX
# ###############################################################
proc read_packets {} {
    set tx_pkg [eval read_from ${::TX_PACKETS_ADDR} "int"]
    set rx_pkg [eval read_from ${::RX_PACKETS_ADDR} "int"]

    return "TX: ${tx_pkg}  &  RX: ${rx_pkg}"
}

# ###############################################################
# Name: read_servo.
#
# Description: Reads information about the servo
#
# Parameters: None.
#
# Return: <integer>: Servo information.
# ###############################################################
proc read_servo {} {
    return [eval read_from ${::SERVO_ADDR} "int"]
}

# ###############################################################
# Name: read_clock_offset.
#
# Description: Reads the current clock offset
#
# Parameters: None.
#
# Return: <integer>: Current clock offset.
# ###############################################################
proc read_clock_offset {} {
    return [eval read_from ${::CLOCK_OFFSET_ADDR} "int"]
}

# ###############################################################
# Name: read_manual_shift.
#
# Description: Reads the manual shift applied in picoseconds.
#
# Parameters: None.
#
# Return: <integer>: Manual shift in picoseconds.
# ###############################################################
proc read_manual_shift {} {
    return [eval read_from ${::MANUAL_SHIFT_ADDR} "int"]
}

# ###############################################################
# Name: read_locked.
#
# Description: Reads if locked to the current frequency.
#
# Parameters: None.
#
# Return: <integer>:
#    - 0: Not Locked.
#	 - 1: Locked.
# ###############################################################
proc read_locked {} {
    return [eval read_from ${::LOCKED_ADDR} "int"]
}

# ###############################################################
# Name: read_calibrated.
#
# Description: Calibration info exchanged
#
# Parameters: None.
#
# Return: <integer>: Calibration info exchanged
# ###############################################################
proc read_calibrated {} {
    return [eval read_from ${::CALIBRATED_ADDR} "int"]
}

# ###############################################################
# Name: read_rtt.
#
# Description: Reads the Round Trip Time.
#
# Parameters: None.
#
# Return: <integer>: Round trip time.
# ###############################################################
proc read_rtt {} {
    return [eval read_from ${::RTT_ADDR} "int"]
}

# ###############################################################
# Name: read_ms_delay.
#
# Description: Reads the Master/slave propagation delay
#
# Parameters: None.
#
# Return: <integer>: Master/slave propagation delay
# ###############################################################
proc read_ms_delay {} {
    return [eval read_from ${::MS_DELAY_ADDR} "int"]
}

# ###############################################################
# Name: read_master_delay.
#
# Description: Reads the master delays.
#
# Parameters: None.
#
# Return: <string>: RX and TX delays.
# ###############################################################
proc read_master_delay {} {
    set txd [eval read_from ${::DELTA_TX_M_ADDR} "int"]
    set rxd [eval read_from ${::DELTA_RX_M_ADDR} "int"]

    return "RX: ${rxd}  &  TX: ${txd}"
}


# ###############################################################
# Name: read_slave_delay.
#
# Description: Reads the slave delays.
#
# Parameters: None.
#
# Return: <string>: RX and TX delays.
# ###############################################################
proc read_slave_delay {} {
    set txd [eval read_from ${::DELTA_TX_S_ADDR} "int"]
    set rxd [eval read_from ${::DELTA_RX_S_ADDR} "int"]

    return "RX: ${rxd}  &  TX: ${txd}"
}


# ###############################################################
# Name: read_semistatic_rx.
#
# Description: RX path semi-static delay: bitslide
#
# Parameters: None.
#
# Return: <integer>: Bitslide.
# ###############################################################
proc read_semistatic_rx {} {
    return [eval read_from ${::SEMISTATIC_RX_ADDR} "int"]
}

# ###############################################################
# Name: read_t24p_delay.
#
# Description: T2 & T4 fixed delay estimation.
#
# Parameters: None.
#
# Return: <integer>: T2 & T4 fixed delay estimation.
# ###############################################################
proc read_t24p_delay {} {
    return [eval read_from ${::T24P_DELAY_ADDR} "int"]
}


# ###############################################################
# Name: read_link_asymmetry.
#
# Description: Calculated asymmetry
#
# Parameters: None.
#
# Return: <integer>: Calculated asymmetry.
# ###############################################################
proc read_link_asymmetry {} {
    return [eval read_from ${::LINK_ASYMMETRY_ADDR} "int"]
}


# ###############################################################
# Name: read_cable_rtt.
#
# Description: Calculated cable Round Trip Time
#
# Parameters: None.
#
# Return: <integer>: Calculated cable Round Trip Time.
# ###############################################################
proc read_cable_rtt {} {
    return [eval read_from ${::CABLE_RTT_ADDR} "int"]
}


# ###############################################################
# Name: read_current_ADDR.
#
# Description: Current applied clock offset.
#
# Parameters: None.
#
# Return: <integer>: Current applied clock offset,
# ###############################################################
proc read_current_offset {} {
    return [eval read_from ${::CURRENT_OFFSET_ADDR} "int"]
}


# ###############################################################
# Name: read_master_clock_class.
#
# Description: Master clock class information.
#
# Parameters: None.
#
# Return: <integer>: Master clock class information.
# ###############################################################
proc read_master_clock_class {} {
    return [eval read_from ${::MASTER_CLOCK_CLASS_ADDR} "int"]
}


# ###############################################################
# Name: read_skew.
#
# Description: Read the clock skew.
#
# Parameters: None.
#
# Return: <integer>: Read the clock skew.
# ###############################################################
proc read_skew {} {
    return [eval read_from ${::SKEW_ADDR} "int"]
}


# ###############################################################
# Name: read_update_count.
#
# Description: Read the clock skew.
#
# Parameters: None.
#
# Return: <integer>: Read the clock skew.
# ###############################################################
proc read_update_count {} {
    return [eval read_from ${::UPDATE_COUNT_ADDR} "int"]
}


# ###############################################################
# Name: read_temperature.
#
# Description: Reads temperature on the FPGA.
#
# Parameters: None.
#
# Return: <integer>: Temperature on the FPGA.
# ###############################################################
proc read_temperature {} {
    return [eval read_from ${::TEMP_ADDR} "int"]
}

# ###############################################################
# Name: read_eeprom_detected.
#
# Description: Reads if the eeprom is detected.
#
# Parameters: None.
#
# Return: <integer>: EEPROM detected:
#    - 0: EEPROM not detected.
#    - 1: EEPROM detected.
# ###############################################################
proc read_eeprom_detected {} {
    return [eval read_from ${::EEPROM_DETECTED_ADDR} "int"]
}

# ###############################################################
# Name: read_utc_offset_valid.
#
# Description: Used to describe the UTC offset when the clock
# is in holdover
#
# Parameters: None.
#
# Return: <integer>: UTC offset valid.
# ###############################################################
proc read_utc_offset_valid {} {
    return [eval read_from ${::HATI_CURRENT_UTC_OFFSET_VALID_ADDR} "int"]
}


# ###############################################################
# Name: read_utc_offset.
#
# Description: Used to describe the UTC offset when the clock
# is in holdover
#
# Parameters: None.
#
# Return: <integer>: UTC offset.
# ###############################################################
proc read_utc_offset {} {
    return [eval read_from ${::HATI_CURRENT_UTC_OFFSET_ADDR} "int"]
}

# ###############################################################
# Name: read_step_removed.
#
# Description: Parameter used to indicate the distance between a
#              clock and the grandmaster clock in the network.
#
# Parameters: None.
#
# Return: <integer>: Steps removed.
# ###############################################################
proc read_step_removed {} {
    return [eval read_from ${::HATI_STEPS_REMOVED_ADDR} "int"]
}


# ###############################################################
# Name: read_servo_state_name.
#
# Description: Reads the servo state name.
#
# Parameters: None.
#
# Return: <string>: Servo state name from SERVO_STATES list.
# ###############################################################
proc read_servo_state_name {} {
    return [lindex ${::SERVO_STATES} [eval read_servo]]
}


# ###############################################################
# Name: read_track_phase.
#
# Description: Reads the track phase.
#
# Parameters: None.
#
# Return: <integer>: Reads phase.
# ###############################################################
proc read_track_phase {} {
    return [eval read_from ${::TRACK_PHASE_ADDR} "int"]
}


# ###############################################################
# Name: read_dac_value.
#
# Description: Reads the DAC value.
#
# Parameters: None.
#
# Return: <integer>: Read DAC value.
# ###############################################################
proc read_dac_value {} {
    return [eval read_from ${::DACVAL_ADDR} "int"]
}


# ###############################################################
# Name: read_clock_id.
#
# Description: Reads the clock ID.
#
# Parameters: None.
#
# Return: <string>: Clock ID.
# ###############################################################
proc read_clock_id {} {
    set clk_id_l [eval read_from [expr ${::CLOCK_ID_ADDR} + 0x00] "lts"]
    set clk_id_h [eval read_from [expr ${::CLOCK_ID_ADDR} + 0x04] "lts"]

    # Create a list with each read character
    set clk_id_aux [concat ${clk_id_l} ${clk_id_h}]

    # Separate Clock ID bytes
    set clk_id1 "[lindex $clk_id_aux 8][lindex $clk_id_aux 9]"
    set clk_id2 "[lindex $clk_id_aux 10][lindex $clk_id_aux 11]"
    set clk_id3 "[lindex $clk_id_aux 12][lindex $clk_id_aux 13]"
    set clk_id4 "[lindex $clk_id_aux 14][lindex $clk_id_aux 15]"
    set clk_id5 "[lindex $clk_id_aux 0][lindex $clk_id_aux 1]"
    set clk_id6 "[lindex $clk_id_aux 2][lindex $clk_id_aux 3]"
    set clk_id7 "[lindex $clk_id_aux 4][lindex $clk_id_aux 5]"
    set clk_id8 "[lindex $clk_id_aux 6][lindex $clk_id_aux 7]"

    return "${clk_id8}:${clk_id7}:${clk_id6}:${clk_id5}:${clk_id4}:${clk_id3}:${clk_id2}:${clk_id1}"
}
