# ################################################################################
# Title: HATI typeutils
# Project: HATI
#
# File       : typeutils.tcl
#
# Licence and Copyright:
# Copyright (c) 2022 Safran Trusted 4D (https://www.safran-group.com/companies/safran-trusted-4d)
#
# You might use, distribute and modify this code and its resulting
# binary form under the terms of the LICENSE.txt provided within the
# repository or project.
#
# If you have not received a copy of the LICENSE.txt along with
# this file please write to info<DOT>spain<AT>orolia<DOT>com and consider that
# this file can not be copied and/or distributed in any forms.
# ################################################################################

# ###############################################################
# Name: toSigned.
#
# Description: 
#    It parses a hexadecimal or unsigned integer value 
#    to a signed interger value. All values are 32-bits long.
# 
# Parameters: 
#    - value: the input to be converted.
#    - intype: (default: hex). Specifies the type of the value
#      input:
#        - hex: The input is a hexadecimal value.
#        - int: The input is a integer value.
#
# Return: The converted signed interger.
# ###############################################################
proc toSigned {value {intype "hex"}} {

    # Checks if the format is hexadecial or integer
    # and parses it to an integer
    if { ${intype} == "hex" } {
        # Parse to interger
        set auxval [scan ${value} %x]
    } else {
        # Already an integer.
        set auxval ${intype}
    }
    
    # Take the sign and magnitude from the value
    set sign 		[expr {( ${auxval} & 0b10000000000000000000000000000000 )}]
    set magnitude  	[expr {( ${auxval} & 0b01111111111111111111111111111111 )}]

    # Check the sign and compute the exponent
    if { ${sign} == 0 } {
        set exponent 0
    } else {
        set exponent [expr -2**31]
    }      

    # Compute the value as signed
    return [expr {${exponent} + ${magnitude}}]
}


# ###############################################################
# Name: toString.
#
# Description: 
#    It parses a hexadecimal or integer value to an ASCII string
# 
# Parameters: 
#    - value: the input to be converted.
#    - intype: (default: hex). Specifies the type of the value
#      input:
#        - hex: The input is a hexadecimal value.
#        - int: The input is a integer value.
#
#     - bits: (default 32): Specifies the length of the value
#       input.
#           - 8: 8 bits input.
#           - 16: 16 bits input.
#           - 32: 32 bits input.
#           - 64: 64 bits input.

#     - endianness: (default: little): Specifies the endianness
#       of the input value.
#           - "little": Little Endian.
#           - "big": Big Endian.
#
# Return: The converted string.
# ###############################################################
proc toString { value {intype "hex"} {bits 32} {endianness "little"} } {

    # Check if the input value is hexadecial or an integer. 
    # If hexadecimal parse it to an integer.
    # (It's a necessary condition parse from hex to int so that you can format as ascii later)

    if { ${intype} == "hex" } {
        set auxval ${value}
    } elseif { ${intype} == "int" } {
        set auxval [scan ${value} %x]
    } else {
        set auxval ${value}
    }

    # Check the input value size then check the endianness
    # and then parses to a string
    if { ${bits} == 32 } {
        if { ${endianness} == "little" } {
            set auxstr [binary format i [scan ${auxval} %x]]
        } elseif { ${endianness} == "big" } {
            set auxstr [binary format I [scan ${auxval} %x]]
        }
    } elseif { ${bits} == 16 } {
        if { ${endianness} == "little" } {
            set auxstr [binary format s [scan ${auxval} %x]]
        } elseif { ${endianness} == "big" } {
            set auxstr [binary format S [scan ${auxval} %x]]
        }
    } elseif { ${bits} == 8 } {
        if { ${endianness} == "little" } {
            set auxstr [binary format c [scan ${auxval} %x]]
        } elseif { ${endianness} == "big" } {
            set auxstr [binary format C [scan ${auxval} %x]]
        }
    } elseif { ${bits} == 64 } {
        if { ${endianness} == "little" } {
            set auxstr [binary format w [scan ${auxval} %x]]
        } elseif { ${endianness} == "big" } {
            set auxstr [binary format W [scan ${auxval} %x]]
        }
    }

    # Remove all the null cahracters and replace them to an empty string.
    set empty_string ""
    regsub -all \000 ${auxstr} ${empty_string} rtnstr

    return ${rtnstr}
}


# ###############################################################
# Name: toList.
#
# Description: 
#    It splits an input value to a list of elements.
# 
# Parameters: 
#    - value: the input to be converted.
#
# Return: The converted list.
# Example: 
#
#   set val [eval toList 12589]
#   puts "List: ${val}"
#   --------------------------------
#   output: List: 1 2 5 8 9
#
# ###############################################################
proc toList {value} {

    # Formats the value as a string, split the charecters and 
    # insert them into a new list.
    return [ list {*}[split [format %s ${value}] {}] ]
}